"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.crsToUnit = crsToUnit;
exports.reasonnableEpsilonForCRS = reasonnableEpsilonForCRS;
exports.assertCrsIsValid = assertCrsIsValid;
exports.crsIsGeographic = crsIsGeographic;
exports.crsIsGeocentric = crsIsGeocentric;
exports.is4326 = is4326;
exports.default = exports.C = exports.UNIT = exports.ellipsoidSizes = void 0;

var _construct2 = _interopRequireDefault(require("@babel/runtime/helpers/construct"));

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var THREE = _interopRequireWildcard(require("three"));

var _proj = _interopRequireDefault(require("proj4"));

var _Ellipsoid = _interopRequireDefault(require("../Math/Ellipsoid"));

/**
 * Generated On: 2015-10-5
 * Class: Coordinates
 * Description: Coordonnées cartographiques
 */
_proj.default.defs('EPSG:4978', '+proj=geocent +datum=WGS84 +units=m +no_defs');

var projectionCache = {};
var dimension = new THREE.Vector2();
var ellipsoidSizes = {
  x: 6378137,
  y: 6378137,
  z: 6356752.3142451793
};
exports.ellipsoidSizes = ellipsoidSizes;
var ellipsoid = new _Ellipsoid.default(ellipsoidSizes);
var UNIT = {
  DEGREE: 1,
  METER: 2
};
exports.UNIT = UNIT;

function _unitFromProj4Unit(projunit) {
  if (projunit === 'degrees') {
    return UNIT.DEGREE;
  } else if (projunit === 'm') {
    return UNIT.METER;
  } else {
    return undefined;
  }
}

function crsToUnit(crs) {
  switch (crs) {
    case 'EPSG:4326':
      return UNIT.DEGREE;

    case 'EPSG:4978':
      return UNIT.METER;

    default:
      {
        var p = _proj.default.defs(crs);

        if (!p) {
          return undefined;
        }

        return _unitFromProj4Unit(p.units);
      }
  }
}

function reasonnableEpsilonForCRS(crs) {
  if (is4326(crs)) {
    return 0.01;
  } else {
    return 0.001;
  }
}

function _crsToUnitWithError(crs) {
  var u = crsToUnit(crs);

  if (crs === undefined || u === undefined) {
    throw new Error("Invalid crs paramater value '".concat(crs, "'"));
  }

  return u;
}

function assertCrsIsValid(crs) {
  _crsToUnitWithError(crs);
}

function crsIsGeographic(crs) {
  return _crsToUnitWithError(crs) != UNIT.METER;
}

function crsIsGeocentric(crs) {
  return _crsToUnitWithError(crs) == UNIT.METER;
}

function _assertIsGeographic(crs) {
  if (!crsIsGeographic(crs)) {
    throw new Error("Can't query crs ".concat(crs, " long/lat"));
  }
}

function _assertIsGeocentric(crs) {
  if (!crsIsGeocentric(crs)) {
    throw new Error("Can't query crs ".concat(crs, " x/y/z"));
  }
}

function instanceProj4(crsIn, crsOut) {
  if (projectionCache[crsIn]) {
    var _p = projectionCache[crsIn];

    if (_p[crsOut]) {
      return _p[crsOut];
    }
  } else {
    projectionCache[crsIn] = {};
  }

  var p = (0, _proj.default)(crsIn, crsOut);
  projectionCache[crsIn][crsOut] = p;
  return p;
}

function is4326(crs) {
  return crs.indexOf('EPSG:4326') == 0;
} // Only support explicit conversions


var cartesian = new THREE.Vector3();

function _convert(coordsIn, newCrs, target) {
  target = target || new Coordinates(newCrs, 0, 0);

  if (newCrs === coordsIn.crs) {
    return target.copy(coordsIn);
  } else {
    if (is4326(coordsIn.crs) && newCrs === 'EPSG:4978') {
      ellipsoid.cartographicToCartesian(coordsIn, cartesian);
      target.set(newCrs, cartesian);
      target._normal = coordsIn._normal;
      target._normalNeedsUpdate = false;
      return target;
    }

    if (coordsIn.crs === 'EPSG:4978' && is4326(newCrs)) {
      ellipsoid.cartesianToCartographic({
        x: coordsIn._values[0],
        y: coordsIn._values[1],
        z: coordsIn._values[2]
      }, target);
      return target;
    }

    if (coordsIn.crs in _proj.default.defs && newCrs in _proj.default.defs) {
      var val0 = coordsIn._values[0];
      var val1 = coordsIn._values[1];
      var crsIn = coordsIn.crs; // there is a bug for converting anything from and to 4978 with proj4
      // https://github.com/proj4js/proj4js/issues/195
      // the workaround is to use an intermediate projection, like EPSG:4326

      if (newCrs == 'EPSG:4978') {
        var p = instanceProj4(crsIn, 'EPSG:4326').forward([val0, val1]);
        target.set('EPSG:4326', p[0], p[1], coordsIn._values[2]);
        return target.as('EPSG:4978', target);
      } else if (coordsIn.crs === 'EPSG:4978') {
        coordsIn.as('EPSG:4326', target);

        var _p2 = instanceProj4(target.crs, newCrs).forward([target._values[0], target._values[1]]);

        target.set(newCrs, _p2[0], _p2[1], target._values[2]);
        return target;
      } else if (is4326(crsIn) && newCrs == 'EPSG:3857') {
        val1 = THREE.Math.clamp(val1, -89.999999, 89.999999);

        var _p3 = instanceProj4(crsIn, newCrs).forward([val0, val1]);

        return target.set(newCrs, _p3[0], _p3[1], coordsIn._values[2]);
      } else {
        // here is the normal case with proj4
        var _p4 = instanceProj4(crsIn, newCrs).forward([val0, val1]);

        return target.set(newCrs, _p4[0], _p4[1], coordsIn._values[2]);
      }
    }

    throw new Error("Cannot convert from crs ".concat(coordsIn.crs, " to ").concat(newCrs));
  }
}
/**
 * Build a Coordinates object, given a [crs]{@link
 * http://inspire.ec.europa.eu/theme/rs} and a number of coordinates value.
 * Coordinates can be in geocentric system, geographic system or an instance of
 * [THREE.Vector3]{@link https://threejs.org/docs/#api/math/Vector3}.
 * If crs = 'EPSG:4326', coordinates must be in geographic system.
 * If crs = 'EPSG:4978', coordinates must be in geocentric system.
 * @constructor
 * @param       {string} crs - Geographic or Geocentric coordinates system.
 * @param       {number|THREE.Vector3} coordinates - The globe coordinates to aim to.
 * @param       {number} coordinates.longitude - Geographic Coordinate longitude
 * @param       {number} coordinates.latitude - Geographic Coordinate latitude
 * @param       {number} coordinates.altitude - Geographic Coordinate altiude
 * @param       {number} coordinates.x - Geocentric Coordinate X
 * @param       {number} coordinates.y - Geocentric Coordinate Y
 * @param       {number} coordinates.z - Geocentric Coordinate Z
 * @example
 * new Coordinates('EPSG:4978', 20885167, 849862, 23385912); //Geocentric coordinates
 * // or
 * new Coordinates('EPSG:4326', 2.33, 48.24, 24999549); //Geographic coordinates
 */


function Coordinates(crs) {
  var _this = this;

  this._values = new Float64Array(3);

  for (var _len = arguments.length, coordinates = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    coordinates[_key - 1] = arguments[_key];
  }

  this.set.apply(this, [crs].concat(coordinates));
  this._normalNeedsUpdate = true;
  this._normal = new THREE.Vector3(); // this._normal = this._normal || computeGeodesicNormal(this);

  Object.defineProperty(this, 'geodesicNormal', {
    configurable: true,
    get: function get() {
      return _this._normalNeedsUpdate ? computeGeodesicNormal(_this) : _this._normal;
    }
  });
}

var planarNormal = new THREE.Vector3(0, 0, 1);

function computeGeodesicNormal(coord) {
  coord._normalNeedsUpdate = false;

  if (is4326(coord.crs)) {
    return ellipsoid.geodeticSurfaceNormalCartographic(coord, coord._normal);
  } // In globe mode (EPSG:4978), we compute the normal.


  if (coord.crs == 'EPSG:4978') {
    return ellipsoid.geodeticSurfaceNormal(coord, coord._normal);
  }

  coord._normal.copy(planarNormal); // In planar mode, normal is the up vector.


  return planarNormal;
}

Coordinates.prototype.set = function (crs) {
  _crsToUnitWithError(crs);

  this.crs = crs;

  for (var _len2 = arguments.length, coordinates = new Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
    coordinates[_key2 - 1] = arguments[_key2];
  }

  if (coordinates.length == 1 && coordinates[0].isVector3) {
    this._values[0] = coordinates[0].x;
    this._values[1] = coordinates[0].y;
    this._values[2] = coordinates[0].z;
  } else {
    for (var i = 0; i < coordinates.length && i < 3; i++) {
      this._values[i] = coordinates[i];
    }

    for (var _i = coordinates.length; _i < 3; _i++) {
      this._values[_i] = 0;
    }
  }

  this._normalNeedsUpdate = true;
  return this;
};

Coordinates.prototype.clone = function (target) {
  var r;

  if (target) {
    Coordinates.call.apply(Coordinates, [target, this.crs].concat((0, _toConsumableArray2.default)(this._values)));
    r = target;
  } else {
    r = (0, _construct2.default)(Coordinates, [this.crs].concat((0, _toConsumableArray2.default)(this._values)));
  }

  if (this._normal) {
    r._normal.copy(this._normal);
  }

  return r;
};

Coordinates.prototype.copy = function (src) {
  this.set.apply(this, [src.crs].concat((0, _toConsumableArray2.default)(src._values)));
  return this;
};
/**
 * Returns the longitude in geographic coordinates. Coordinates must be in geographic system (can be converted by using {@linkcode as()} ).
 * @example
 *
 * const position = { longitude: 2.33, latitude: 48.24, altitude: 24999549 };
 * const coordinates = new Coordinates('EPSG:4326', position.longitude, position.latitude, position.altitude); // Geographic system
 * coordinates.longitude(); // Longitude in geographic system
 * // returns 2.33
 *
 * // or
 *
 * const position = { x: 20885167, y: 849862, z: 23385912 };
 * const coords = new Coordinates('EPSG:4978', position.x, position.y, position.z);  // Geocentric system
 * const coordinates = coords.as('EPSG:4326');  // Geographic system
 * coordinates.longitude(); // Longitude in geographic system
 * // returns 2.330201911389028
 *
 * @return     {number} - The longitude of the position.
 */


Coordinates.prototype.longitude = function () {
  _assertIsGeographic(this.crs);

  return this._values[0];
};
/**
 * Returns the latitude in geographic coordinates. Coordinates must be in geographic system (can be converted by using {@linkcode as()} ).
 * @example
 *
 * const position = { longitude: 2.33, latitude: 48.24, altitude: 24999549 };
 * const coordinates = new Coordinates('EPSG:4326', position.longitude, position.latitude, position.altitude); // Geographic system
 * coordinates.latitude(); // Latitude in geographic system
 * // returns : 48.24
 *
 * // or
 *
 * const position = { x: 20885167, y: 849862, z: 23385912 };
 * const coords = new Coordinates('EPSG:4978', position.x, position.y, position.z);  // Geocentric system
 * const coordinates = coords.as('EPSG:4326');  // Geographic system
 * coordinates.latitude(); // Latitude in geographic system
 * // returns : 48.24830764643365
 *
 * @return     {number} - The latitude of the position.
 */


Coordinates.prototype.latitude = function () {
  return this._values[1];
};
/**
 * Returns the altitude in geographic coordinates. Coordinates must be in geographic system (can be converted by using {@linkcode as()} ).
 * @example
 *
 * const position = { longitude: 2.33, latitude: 48.24, altitude: 24999549 };
 * const coordinates = new Coordinates('EPSG:4326', position.longitude, position.latitude, position.altitude); // Geographic system
 * coordinates.altitude(); // Altitude in geographic system
 * // returns : 24999549
 *
 * // or
 *
 * const position = { x: 20885167, y: 849862, z: 23385912 };
 * const coords = new Coordinates('EPSG:4978', position.x, position.y, position.z);  // Geocentric system
 * const coordinates = coords.as('EPSG:4326');  // Geographic system
 * coordinates.altitude(); // Altitude in geographic system
 * // returns : 24999548.046711832
 *
 * @return     {number} - The altitude of the position.
 */


Coordinates.prototype.altitude = function () {
  _assertIsGeographic(this.crs);

  return this._values[2];
};
/**
 * Set the altiude.
 * @example coordinates.setAltitude(number)
 * @param      {number} - Set the altitude.
 */


Coordinates.prototype.setAltitude = function (altitude) {
  _assertIsGeographic(this.crs);

  this._values[2] = altitude;
};
/**
 * Returns the longitude in geocentric coordinates. Coordinates must be in geocentric system (can be converted by using {@linkcode as()} ).
 * @example
 *
 * const position = { x: 20885167, y: 849862, z: 23385912 };
 * const coordinates = new Coordinates('EPSG:4978', position.x, position.y, position.z);  // Geocentric system
 * coordinates.x();  // Geocentric system
 * // returns : 20885167
 *
 * // or
 *
 * const position = { longitude: 2.33, latitude: 48.24, altitude: 24999549 };
 * const coords = new Coordinates('EPSG:4326', position.longitude, position.latitude, position.altitude); // Geographic system
 * const coordinates = coords.as('EPSG:4978'); // Geocentric system
 * coordinates.x(); // Geocentric system
 * // returns : 20888561.0301258
 *
 * @return     {number} - The longitude of the position.
 */


Coordinates.prototype.x = function () {
  _assertIsGeocentric(this.crs);

  return this._values[0];
};
/**
 * Returns the latitude in geocentric coordinates. Coordinates must be in geocentric system (can be converted by using {@linkcode as()} ).
 * @example
 *
 * const position = { x: 20885167, y: 849862, z: 23385912 };
 * const coordinates = new Coordinates('EPSG:4978', position.x, position.y, position.z);  // Geocentric system
 * coordinates.y();  // Geocentric system
 * // returns : 849862
 *
 * // or
 *
 * const position = { longitude: 2.33, latitude: 48.24, altitude: 24999549 };
 * const coords = new Coordinates('EPSG:4326', position.longitude, position.latitude, position.altitude); // Geographic system
 * const coordinates = coords.as('EPSG:4978'); // Geocentric system
 * coordinates.y(); // Geocentric system
 * // returns : 849926.376770819
 *
 * @return     {number} - The latitude of the position.
 */


Coordinates.prototype.y = function () {
  _assertIsGeocentric(this.crs);

  return this._values[1];
};
/**
 * Returns the altitude in geocentric coordinates. Coordinates must be in geocentric system (can be converted by using {@linkcode as()} ).
 * @example
 *
 * const position = { x: 20885167, y: 849862, z: 23385912 };
 * const coordinates = new Coordinates('EPSG:4978', position.x, position.y, position.z);  // Geocentric system
 * coordinates.z();  // Geocentric system
 * // returns : 23385912
 *
 * // or
 *
 * const position = { longitude: 2.33, latitude: 48.24, altitude: 24999549 };
 * const coords = new Coordinates('EPSG:4326', position.longitude, position.latitude, position.altitude); // Geographic system
 * const coordinates = coords.as('EPSG:4978'); // Geocentric system
 * coordinates.z(); // Geocentric system
 * // returns : 23382883.536591515
 *
 * @return     {number} - The altitude of the position.
 */


Coordinates.prototype.z = function () {
  _assertIsGeocentric(this.crs);

  return this._values[2];
};
/**
 * Returns a position in cartesian coordinates. Coordinates must be in geocentric system (can be converted by using {@linkcode as()} ).
 * @example
 *
 * const position = { x: 20885167, y: 849862, z: 23385912 };
 * const coordinates = new Coordinates('EPSG:4978', position.x, position.y, position.z);  // Geocentric system
 * coordinates.xyz();  // Geocentric system
 * // returns : Vector3
 * // x: 20885167
 * // y: 849862
 * // z: 23385912
 *
 * // or
 *
 * const position = { longitude: 2.33, latitude: 48.24, altitude: 24999549 };
 * const coords = new Coordinates('EPSG:4326', position.longitude, position.latitude, position.altitude); // Geographic system
 * const coordinates = coords.as('EPSG:4978'); // Geocentric system
 * coordinates.xyz(); // Geocentric system
 * // returns : Vector3
 * // x: 20885167
 * // y: 849862
 * // z: 23385912
 *
 * @return     {Position} - position
 */


Coordinates.prototype.xyz = function (target) {
  _assertIsGeocentric(this.crs);

  var v = target || new THREE.Vector3();
  v.fromArray(this._values);
  return v;
};
/**
 * Returns coordinates in the wanted [CRS]{@link http://inspire.ec.europa.eu/theme/rs}.
 * @example
 *
 * const position = { longitude: 2.33, latitude: 48.24, altitude: 24999549 };
 * const coords = new Coordinates('EPSG:4326', position.longitude, position.latitude, position.altitude); // Geographic system
 * const coordinates = coords.as('EPSG:4978'); // Geocentric system
 *
 * // or
 *
 * const position = { x: 20885167, y: 849862, z: 23385912 };
 * const coords = new Coordinates('EPSG:4978', position.x, position.y, position.z);  // Geocentric system
 * const coordinates = coords.as('EPSG:4326');  // Geographic system
 *
 * //or
 *
 * new Coordinates('EPSG:4326', longitude: 2.33, latitude: 48.24, altitude: 24999549).as('EPSG:4978'); // Geocentric system
 *
 * // or
 *
 * new Coordinates('EPSG:4978', x: 20885167, y: 849862, z: 23385912).as('EPSG:4326'); // Geographic system
 *
 * @param      {string} - [crs]{@link http://inspire.ec.europa.eu/theme/rs} : Geocentric (ex: 'EPSG:4326') or Geographic (ex: 'EPSG:4978').
 * @return     {Position} - position
 */


Coordinates.prototype.as = function (crs, target) {
  if (crs === undefined || crsToUnit(crs) === undefined) {
    throw new Error("Invalid crs paramater value '".concat(crs, "'"));
  }

  return _convert(this, crs, target);
};
/**
 * Returns the normalized offset from top-left in extent of this Coordinates
 * e.g: extent.center().offsetInExtent(extent) would return (0.5, 0.5).
 * @param {Extent} extent
 * @param {Vector2} target optional Vector2 target. If not present a new one will be created
 * @return {Vector2} normalized offset in extent
 */


Coordinates.prototype.offsetInExtent = function (extent, target) {
  if (this.crs != extent.crs) {
    throw new Error('unsupported mix');
  }

  extent.dimensions(dimension);
  var x = crsIsGeocentric(this.crs) ? this.x() : this.longitude();
  var y = crsIsGeocentric(this.crs) ? this.y() : this.latitude();
  var originX = (x - extent.west) / dimension.x;
  var originY = (extent.north - y) / dimension.y;
  target = target || new THREE.Vector2();
  target.set(originX, originY);
  return target;
};

var C = {
  /**
   * Return a Coordinates object from a position object. The object just
   * needs to have x, y, z properties.
   *
   * @param {string} crs - The crs of the original position
   * @param {Object} position - the position to transform
   * @param {number} position.x - the x component of the position
   * @param {number} position.y - the y component of the position
   * @param {number} position.z - the z component of the position
   * @return {Coordinates}
   */
  EPSG_4326: function () {
    for (var _len3 = arguments.length, args = new Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
      args[_key3] = arguments[_key3];
    }

    return (0, _construct2.default)(Coordinates, ['EPSG:4326'].concat(args));
  }
};
exports.C = C;
var _default = Coordinates;
exports.default = _default;