"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createGlobeLayer = createGlobeLayer;
exports.default = exports.GLOBE_VIEW_EVENTS = void 0;

var THREE = _interopRequireWildcard(require("three"));

var _View = _interopRequireWildcard(require("../View"));

var _MainLoop = require("../MainLoop");

var _ColorLayersOrdering = require("../../Renderer/ColorLayersOrdering");

var _GlobeControls = _interopRequireDefault(require("../../Controls/GlobeControls"));

var _GlobeLayer = _interopRequireDefault(require("./Globe/GlobeLayer"));

var _Atmosphere = _interopRequireDefault(require("./Globe/Atmosphere"));

var _Coordinates = _interopRequireWildcard(require("../Geographic/Coordinates"));

/**
 * Fires when the view is completely loaded. Controls and view's functions can be called then.
 * @event GlobeView#initialized
 * @property target {view} dispatched on view
 * @property type {string} initialized
 */

/**
 * Fires when a layer is added
 * @event GlobeView#layer-added
 * @property layerId {string} the id of the layer
 * @property target {view} dispatched on view
 * @property type {string} layers-added
 */

/**
 * Fires when a layer is removed
 * @event GlobeView#layer-removed
 * @property layerId {string} the id of the layer
 * @property target {view} dispatched on view
 * @property type {string} layers-added
 */

/**
 * Fires when the layers oder has changed
 * @event GlobeView#layers-order-changed
 * @property new {object}
 * @property new.sequence {array}
 * @property new.sequence.0 {number} the new layer at position 0
 * @property new.sequence.1 {number} the new layer at position 1
 * @property new.sequence.2 {number} the new layer at position 2
 * @property previous {object}
 * @property previous.sequence {array}
 * @property previous.sequence.0 {number} the previous layer at position 0
 * @property previous.sequence.1 {number} the previous layer at position 1
 * @property previous.sequence.2 {number} the previous layer at position 2
 * @property target {view} dispatched on view
 * @property type {string} layers-order-changed
 */

/**
 * Globe's EVENT
 * @property GLOBE_INITIALIZED {string} emit one time when globe is initialized
 * @property LAYER_ADDED {string} emit when layer id added in viewer
 * @property LAYER_REMOVED {string} emit when layer id removed in viewer
 * @property COLOR_LAYERS_ORDER_CHANGED {string} emit when  color layers order change
 */
var GLOBE_VIEW_EVENTS = {
  GLOBE_INITIALIZED: 'initialized',
  LAYER_ADDED: 'layer-added',
  LAYER_REMOVED: 'layer-removed',
  COLOR_LAYERS_ORDER_CHANGED: _ColorLayersOrdering.COLOR_LAYERS_ORDER_CHANGED
};
exports.GLOBE_VIEW_EVENTS = GLOBE_VIEW_EVENTS;

function createGlobeLayer(id) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  console.warn('createGlobeLayer is deprecated, use the GlobeLayer class instead.');
  return new _GlobeLayer.default(id, options.object3d, options);
}
/**
 * Creates a view of a globe.
 *
 * @constructor
 *
 * @example
 * var viewerDiv = document.getElementById('viewerDiv');
 * var position = new itowns.Coordinates('WGS84', 2.35, 48.8, 25e6);
 * var view = new itowns.GlobeView(viewerDiv, position);
 *
 * @example
 * var viewerDiv = document.getElementById('viewerDiv');
 * var position = { longitude: 2.35, latitude: 48.8, altitude: 25e6 };
 * var view = new itowns.GlobeView(viewerDiv, position);
 *
 * @param {HTMLDivElement} viewerDiv - Where to attach the view and display it
 * in the DOM.
 * @param {object|Coordinates} coordCarto - An object containing three
 * properties: longitude, latitude and altitude. It will help placing the camera
 * on the globe at the creation.
 * @param {object=} options - See options of {@link View}.
 */


function GlobeView(viewerDiv, coordCarto) {
  var _this = this;

  var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  THREE.Object3D.DefaultUp.set(0, 0, 1); // Setup View

  _View.default.call(this, 'EPSG:4978', viewerDiv, options); // Configure camera


  var positionCamera;

  if (coordCarto instanceof _Coordinates.default) {
    positionCamera = coordCarto.as('EPSG:4326');
  } else {
    positionCamera = new _Coordinates.C.EPSG_4326(coordCarto.longitude, coordCarto.latitude, coordCarto.altitude);
  }

  this.camera.camera3D.near = Math.max(15.0, 0.000002352 * _Coordinates.ellipsoidSizes.x);
  this.camera.camera3D.far = _Coordinates.ellipsoidSizes.x * 10;
  var tileLayer = new _GlobeLayer.default('globe', options.object3d, options);
  var sun = new THREE.DirectionalLight();
  sun.position.set(-0.5, 0, 1);
  sun.updateMatrixWorld(true);
  tileLayer.object3d.add(sun);
  this.addLayer(tileLayer); // Configure controls

  var positionTargetCamera = positionCamera.clone();
  positionTargetCamera.setAltitude(0);

  if (options.noControls) {
    this.camera.setPosition(positionCamera);
    this.camera.camera3D.lookAt(positionTargetCamera.as('EPSG:4978').xyz());
  } else {
    this.controls = new _GlobeControls.default(this, positionTargetCamera, positionCamera.altitude(), _Coordinates.ellipsoidSizes.x);
    this.controls.handleCollision = typeof options.handleCollision !== 'undefined' ? options.handleCollision : true;
  }

  this._fullSizeDepthBuffer = null;
  this.addFrameRequester(_MainLoop.MAIN_LOOP_EVENTS.BEFORE_RENDER, function () {
    if (_this._fullSizeDepthBuffer != null) {
      // clean depth buffer
      _this._fullSizeDepthBuffer = null;
    }
  });
  this.tileLayer = tileLayer;
  this.addLayer(new _Atmosphere.default());

  var fn = function () {
    _this.removeEventListener(_View.VIEW_EVENTS.LAYERS_INITIALIZED, fn);

    _this.dispatchEvent({
      type: GLOBE_VIEW_EVENTS.GLOBE_INITIALIZED
    });
  }; // GlobeView needs this.camera.resize to set perpsective matrix camera


  this.camera.resize(viewerDiv.clientWidth, viewerDiv.clientHeight);
  this.addEventListener(_View.VIEW_EVENTS.LAYERS_INITIALIZED, fn);
}

GlobeView.prototype = Object.create(_View.default.prototype);
GlobeView.prototype.constructor = GlobeView;

GlobeView.prototype.addLayer = function (layer) {
  if (!layer) {
    return new Promise(function (resolve, reject) {
      return reject(new Error('layer is undefined'));
    });
  }

  if (layer.isColorLayer) {
    var colorLayerCount = this.getLayers(function (l) {
      return l.isColorLayer;
    }).length;
    layer.sequence = colorLayerCount;
  } else if (layer.isElevationLayer) {
    if (layer.source.isWMTSSource && layer.source.tileMatrixSet !== 'WGS84G') {
      throw new Error('Only WGS84G tileMatrixSet is currently supported for WMTS elevation layers');
    }
  }

  var layerId = layer.id;

  var layerPromise = _View.default.prototype.addLayer.call(this, layer, this.tileLayer);

  this.dispatchEvent({
    type: GLOBE_VIEW_EVENTS.LAYER_ADDED,
    layerId: layerId
  });
  return layerPromise;
};
/**
 * Removes a specific imagery layer from the current layer list. This removes layers inserted with attach().
 * @example
 * view.removeLayer('layerId');
 * @param      {string}   layerId      The identifier
 * @return     {boolean}
 */


GlobeView.prototype.removeLayer = function (layerId) {
  if (_View.default.prototype.removeLayer.call(this, layerId)) {
    this.dispatchEvent({
      type: GLOBE_VIEW_EVENTS.LAYER_REMOVED,
      layerId: layerId
    });
    return true;
  }
};

var _default = GlobeView;
exports.default = _default;