"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _Coordinates = _interopRequireDefault(require("../Core/Geographic/Coordinates"));

var _Extent = _interopRequireDefault(require("../Core/Geographic/Extent"));

function readCRS(json) {
  if (json.crs) {
    if (json.crs.type.toLowerCase() == 'epsg') {
      return "EPSG:".concat(json.crs.properties.code);
    } else if (json.crs.type.toLowerCase() == 'name') {
      var epsgIdx = json.crs.properties.name.toLowerCase().indexOf('epsg:');

      if (epsgIdx >= 0) {
        // authority:version:code => EPSG:[...]:code
        var codeStart = json.crs.properties.name.indexOf(':', epsgIdx + 5);

        if (codeStart > 0) {
          return "EPSG:".concat(json.crs.properties.name.substr(codeStart + 1));
        }
      }
    }

    throw new Error("Unsupported CRS type '".concat(json.crs, "'"));
  } // assume default crs


  return 'EPSG:4326';
}

var coordIn = new _Coordinates.default('EPSG:4978', 0, 0, 0);
var coordOut = new _Coordinates.default('EPSG:4978', 0, 0, 0);

function readCoordinates(crsIn, crsOut, coordinates, extent, target, normals, size) {
  var overrideAltitudeInToZero = arguments.length > 7 && arguments[7] !== undefined ? arguments[7] : false;
  // coordinates is a list of pair [[x1, y1], [x2, y2], ..., [xn, yn]]
  var offset = 0; // let cIn = coordIn;

  var cOut = coordOut;

  if (target) {
    offset = target.length;
    var count = coordinates.length * size;
    target.length += count;

    if (normals) {
      normals.length += count;
    }
  }

  target = target || new Array(coordinates.length);
  var z = 0;
  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = coordinates[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var pair = _step.value;

      if (!overrideAltitudeInToZero && size == 3 && typeof pair[2] == 'number') {
        z = pair[2];
      }

      coordIn.set(crsIn, pair[0], pair[1], z);

      if (crsIn !== crsOut) {
        coordIn.as(crsOut, coordOut);
      } else {
        cOut = coordIn;
      }

      if (normals) {
        cOut.geodesicNormal.toArray(normals, offset);
      }

      target[offset] = cOut._values[0];
      target[offset + 1] = cOut._values[1];

      if (size == 3) {
        target[offset + 2] = cOut._values[2];
      } // expand extent if present


      if (extent) {
        if (extent.crs == crsIn) {
          extent.expandByCoordinates(coordIn);
        } else {
          extent.expandByCoordinates(cOut);
        }
      }

      offset += size;
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return != null) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  return target;
} // Helper struct that returns an object { type: "", coordinates: [...], extent}:
// - type is the geom type
// - Coordinates is an array of Coordinate
// - extent is optional, it's coordinates's extent
// Multi-* geometry types are merged in one.


var coords = new _Coordinates.default('EPSG:4978', 0, 0, 0); // filter with the first point

var firstPtIsOut = function (extent, aCoords, crs) {
  var first = aCoords[0];
  coords.set(crs, first[0], first[1], 0);
  return !extent.isPointInside(coords);
};

var GeometryToCoordinates = {
  point: function point(feature, crsIn, crsOut, coordsIn, filteringExtent, options) {
    // filtering
    if (filteringExtent && firstPtIsOut(filteringExtent, coordsIn, crsIn)) {
      return;
    }

    var extent = options.buildExtent ? new _Extent.default(crsOut, Infinity, -Infinity, Infinity, -Infinity) : undefined;
    var offset = feature.vertices.length / feature.size;
    readCoordinates(crsIn, crsOut, coordsIn, extent, feature.vertices, feature.normals, feature.size, options.overrideAltitudeInToZero);
    feature.geometry.push({
      extent: extent,
      indices: [{
        offset: offset,
        count: 1
      }]
    });
    return feature;
  },
  polygon: function polygon(feature, crsIn, crsOut, coordsIn, filteringExtent, options) {
    // filtering
    if (filteringExtent && firstPtIsOut(filteringExtent, coordsIn[0], crsIn)) {
      return;
    }

    var extent = options.buildExtent ? new _Extent.default(crsOut, Infinity, -Infinity, Infinity, -Infinity) : undefined;
    var offset = feature.vertices.length / feature.size; // read contour first

    readCoordinates(crsIn, crsOut, coordsIn[0], extent, feature.vertices, feature.normals, feature.size, options.overrideAltitudeInToZero);
    var indices = [{
      offset: offset,
      count: coordsIn[0].length
    }];
    offset += coordsIn[0].length; // Then read optional holes

    for (var i = 1; i < coordsIn.length; i++) {
      readCoordinates(crsIn, crsOut, coordsIn[i], extent, feature.vertices, feature.normals, feature.size, options.overrideAltitudeInToZero);
      var count = coordsIn[i].length;
      indices.push({
        offset: offset,
        count: count
      });
      offset += count;
    }

    feature.geometry.push({
      extent: extent,
      indices: indices
    });
    return feature;
  },
  lineString: function lineString(feature, crsIn, crsOut, coordsIn, filteringExtent, options) {
    // filtering
    if (filteringExtent && firstPtIsOut(filteringExtent, coordsIn, crsIn)) {
      return;
    }

    var extent = options.buildExtent ? new _Extent.default(crsOut, Infinity, -Infinity, Infinity, -Infinity) : undefined;
    var offset = feature.vertices.length / feature.size;
    readCoordinates(crsIn, crsOut, coordsIn, extent, feature.vertices, feature.normals, feature.size, options.overrideAltitudeInToZero);
    var indices = [{
      offset: offset,
      count: feature.vertices.length / feature.size - offset
    }];
    feature.geometry.push({
      extent: extent,
      indices: indices
    });
    return feature;
  },
  multi: function multi(type, feature, crsIn, crsOut, coordsIn, filteringExtent, options) {
    var _iteratorNormalCompletion2 = true;
    var _didIteratorError2 = false;
    var _iteratorError2 = undefined;

    try {
      for (var _iterator2 = coordsIn[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
        var _coords = _step2.value;
        this[type](feature, crsIn, crsOut, _coords, filteringExtent, options);
      }
    } catch (err) {
      _didIteratorError2 = true;
      _iteratorError2 = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion2 && _iterator2.return != null) {
          _iterator2.return();
        }
      } finally {
        if (_didIteratorError2) {
          throw _iteratorError2;
        }
      }
    }

    return feature;
  }
};

function readGeometry(type, feature, crsIn, crsOut, geometry, filteringExtent, options) {
  if (geometry.length == 0) {
    return;
  }

  switch (type) {
    case 'point':
      return GeometryToCoordinates.point(feature, crsIn, crsOut, [geometry], filteringExtent, options);

    case 'multipoint':
      return GeometryToCoordinates.multi('point', feature, crsIn, crsOut, geometry, filteringExtent, options);

    case 'linestring':
      return GeometryToCoordinates.lineString(feature, crsIn, crsOut, geometry, filteringExtent, options);

    case 'multilinestring':
      return GeometryToCoordinates.multi('lineString', feature, crsIn, crsOut, geometry, filteringExtent, options);

    case 'polygon':
      return GeometryToCoordinates.polygon(feature, crsIn, crsOut, geometry, filteringExtent, options);

    case 'multipolygon':
      return GeometryToCoordinates.multi('polygon', feature, crsIn, crsOut, geometry, filteringExtent, options);

    case 'geometrycollection':
    default:
      throw new Error("Unhandled geometry type ".concat(feature.type));
  }
}

function mergeType(type) {
  switch (type) {
    case 'point':
    case 'multipoint':
      return 'multipoint';

    case 'linestring':
    case 'multilinestring':
      return 'multilinestring';

    case 'polygon':
    case 'multipolygon':
      return 'multipolygon';

    case 'geometrycollection':
    default:
      throw new Error("Unhandled geometry type ".concat(type));
  }
}

var keyProperties = ['type', 'geometry', 'properties'];

function readFeature(crsIn, crsOut, json, filteringExtent, options) {
  var featureMerge = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : {};

  if (options.filter && !options.filter(json.properties, json.geometry)) {
    return;
  }

  var jsonType = json.geometry.type.toLowerCase();
  var type = options.mergeFeatures ? mergeType(jsonType) : jsonType;
  var feature = featureMerge[type] || {
    type: type,
    geometry: [],
    vertices: [],
    normals: options.withNormal ? [] : undefined,
    crs: crsOut,
    size: options.withAltitude ? 3 : 2
  };
  var offset = feature.geometry.length;
  readGeometry(jsonType, feature, crsIn, crsOut, json.geometry.coordinates, filteringExtent, options);

  if (feature.geometry.length == offset) {
    return;
  }

  var properties = json.properties || {}; // copy other properties

  var _arr = Object.keys(json);

  for (var _i = 0; _i < _arr.length; _i++) {
    var key = _arr[_i];

    if (!keyProperties.includes(key.toLowerCase())) {
      properties[key] = json[key];
    }
  }

  for (var i = offset; i < feature.geometry.length; i++) {
    var g = feature.geometry[i];

    if (options.buildExtent) {
      feature.extent = feature.extent || g.extent;
      feature.extent.union(g.extent);
    }

    g.properties = properties;
  }

  return feature;
}

var mergeExtent = function (res, extent) {
  if (res.extent) {
    res.extent.union(extent);
  } else {
    res.extent = extent.clone();
  }
};

var mergesType = ['multipolygon', 'multilinestring', 'multipoint'];

function readFeatures(crsIn, crsOut, features, filteringExtent, options) {
  var res = {
    features: []
  };
  var featuresMerge = {};

  if (options.mergeFeatures) {
    for (var _i2 = 0; _i2 < mergesType.length; _i2++) {
      var type = mergesType[_i2];
      featuresMerge[type] = {
        type: type,
        geometry: [],
        vertices: [],
        normals: options.withNormal ? [] : undefined,
        crs: crsOut,
        size: options.withAltitude ? 3 : 2
      };
    }
  }

  var _iteratorNormalCompletion3 = true;
  var _didIteratorError3 = false;
  var _iteratorError3 = undefined;

  try {
    for (var _iterator3 = features[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
      var feature = _step3.value;

      var _f = readFeature(crsIn, crsOut, feature, filteringExtent, options, featuresMerge);

      if (_f && !options.mergeFeatures) {
        if (options.buildExtent) {
          mergeExtent(res, _f.extent);
        }

        res.features.push(_f);
      }
    }
  } catch (err) {
    _didIteratorError3 = true;
    _iteratorError3 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion3 && _iterator3.return != null) {
        _iterator3.return();
      }
    } finally {
      if (_didIteratorError3) {
        throw _iteratorError3;
      }
    }
  }

  if (options.mergeFeatures) {
    for (var _i3 = 0; _i3 < mergesType.length; _i3++) {
      var _type = mergesType[_i3];
      var f = featuresMerge[_type];

      if (f.geometry.length) {
        f.vertices.crs = crsOut;
        res.features.push(f);

        if (options.buildExtent) {
          mergeExtent(res, f.extent);
        }
      }
    }
  }

  res.crs = crsOut;
  res.isFeature = true;
  return res;
}
/**
 * The GeoJsonParser module provide a [parse]{@link module:GeoJsonParser.parse}
 * method that takes a GeoJSON in and gives an object formatted for iTowns
 * containing all necessary informations to display this GeoJSON.
 *
 * @module GeoJsonParser
 */


var _default = {
  /**
   * @typedef {Object} GeoJsonParserOptions
   * @property {string} crsOut - The CRS to convert the input coordinates
   * to.
   * @property {string} crsIn - Override the data CRS.
   * @property {Extent} [filteringExtent] - Optional filter to reject
   * features outside of this extent.
   * @property {boolean} [buildExtent=false] - If true the geometry will
   * have an extent property containing the area covered by the geom
   * @property {function} [filter] - Filter function to remove features
   * @property {boolean} [mergeFeatures=true] - If true all geometries are merged by type and multi-type
   * @property {boolean} [withNormal=true] - If true each coordinate normal is computed
   * @property {boolean} [withAltitude=true] - If true each coordinate altitude is kept
   * @property {boolean} [overrideAltitudeInToZero=false] - If true, the altitude of the source data isn't taken into account for 3D geometry convertions.
   * the altitude will be override to 0. This can be useful if you don't have a DEM or provide a new one when converting (with Layer.convert).
   */

  /**
   * Similar to the geometry of a feature in a GeoJSON, but adapted to iTowns.
   * The difference is that coordinates are stored in unique Array of number
   *
   * @typedef FeatureGeometry
   * @type {Object}
   *
   * @property {Extent} extent - The 2D extent containing all the points
   * composing the geometry.
   * @property {?Object[]} indices - If this geometry is a
   * <code>linestring</code> or a <code>polygon</code>, contains the indices
   * that define the geometry. Objects stored in this array have two
   * properties, an <code>offset</code> and a <code>count</code>. The offset
   * is related to the overall number of vertices in the Feature.
   *
   * @property {Object} properties - Properties of the geometry. It can be
   * anything specified in the GeoJSON under the <code>properties</code>
   * property.
   */

  /**
   * Similar to a feature in a GeoJSON, but adapted to iTowns.
   *
   * @typedef Feature
   * @type {Object}
   *
   * @property {string} type - Geometry type, can be <code>point</code>,
   * <code>multipoint</code>, <code>linestring</code>,
   * <code>multilinestring</code>, <code>polygon</code> or
   * <code>multipolygon</code>.
   * @property {number[]} vertices - All the vertices of the geometry.
   * @property {number[]} normals - All the normals of the geometry.
   * @property {number} size - the number of values of the array that should be associated with a coordinates.
   * The size is 3 with altitude and 2 without altitude.
   * @property {string} crs - Geographic or Geocentric coordinates system.
   * @property {FeatureGeometry[]} geometry - The feature's geometry, as an
   * array of [FeatureGeometry]{@link module:GeoJsonParser~FeatureGeometry}.
   * @property {Extent?} extent - The 2D extent containing all the geometries
   * composing the feature.
   */

  /**
   * An object regrouping a list of [features]{@link
   * module:GeoJsonParser~Feature} and the extent of this collection.
   *
   * @typedef FeatureCollection
   * @type {Object}
   *
   * @property {Feature[]} features - The array of features composing the
   * collection.
   * @property {Extent?} extent - The 2D extent containing all the features
   * composing the collection.
   * @property {string} crs - Geographic or Geocentric coordinates system.
   * @property {boolean} isFeature - Used to check whether this is FeatureCollection.
   */

  /**
   * Parse a GeoJSON file content and return a [FeatureCollection]{@link
   * module:GeoJsonParser~FeatureCollection}.
   *
   * @param {string} json - The GeoJSON file content to parse.
   * @param {GeoJsonParser~GeoJsonParserOptions} options - Options controlling
   * the parsing.
   *
   * @return {Promise} A promise resolving with a [FeatureCollection]{@link
   * module:GeoJsonParser~FeatureCollection}.
   */
  parse: function parse(json) {
    var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    var crsOut = options.crsOut;
    var filteringExtent = options.filteringExtent;

    if (typeof json === 'string') {
      json = JSON.parse(json);
    }

    options.crsIn = options.crsIn || readCRS(json);
    options.mergeFeatures = options.mergeFeatures == undefined ? true : options.mergeFeatures;
    options.withNormal = options.withNormal == undefined ? true : options.withNormal;
    options.withAltitude = options.withAltitude == undefined ? true : options.withAltitude;

    switch (json.type.toLowerCase()) {
      case 'featurecollection':
        return Promise.resolve(readFeatures(options.crsIn, crsOut, json.features, filteringExtent, options));

      case 'feature':
        return Promise.resolve(readFeatures(options.crsIn, crsOut, [json], filteringExtent, options));

      default:
        throw new Error("Unsupported GeoJSON type: '".concat(json.type));
    }
  }
};
exports.default = _default;