"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _pbf = _interopRequireDefault(require("pbf"));

var _vectorTile = require("@mapbox/vector-tile");

var _GeoJsonParser = _interopRequireDefault(require("./GeoJsonParser"));

var _Coordinates = _interopRequireDefault(require("../Core/Geographic/Coordinates"));

// This part is freely adapted from vector-tile-js
// https://github.com/mapbox/vector-tile-js/blob/master/lib/vectortilefeature.js
function signedArea(ring) {
  var sum = 0;

  for (var i = 0, len = ring.length, j = len - 1, p1, p2; i < len; j = i++) {
    p1 = ring[i];
    p2 = ring[j];
    sum += (p2.x - p1.x) * (p1.y + p2.y);
  }

  return sum;
}

function classifyRings(rings) {
  var len = rings.length;

  if (len <= 1) {
    return [rings];
  }

  var polygons = [];
  var polygon;
  var ccw;

  for (var i = 0; i < len; i++) {
    var area = signedArea(rings[i]);

    if (area === 0) {
      continue;
    }

    if (ccw === undefined) {
      ccw = area < 0;
    }

    if (ccw === area < 0) {
      if (polygon) {
        polygons.push(polygon);
      }

      polygon = [rings[i]];
    } else {
      polygon.push(rings[i]);
    }
  }

  if (polygon) {
    polygons.push(polygon);
  }

  return polygons;
}

var VectorTileFeature = {
  types: ['Unknown', 'Point', 'LineString', 'Polygon']
}; // EPSG:3857
// WGS84 bounds [-180.0, -85.06, 180.0, 85.06] (https://epsg.io/3857)

var coord = new _Coordinates.default('EPSG:4326', 180, 85.06);
coord.as('EPSG:3857', coord); // Get bound dimension in 'EPSG:3857'

var sizeX = coord._values[0] * 2;
var sizeY = coord._values[1] * 2;

function project(line, ox, oy, size) {
  for (var j = 0; j < line.length; j++) {
    var p = line[j];
    line[j] = [((p.x + ox) / size - 0.5) * sizeX, (0.5 - (p.y + oy) / size) * sizeY];
  }
}

function toGeoJSON(x, y, z) {
  var size = this.extent * Math.pow(2, z);
  var x0 = this.extent * x;
  var y0 = this.extent * y;
  var coords = this.loadGeometry();
  var type = VectorTileFeature.types[this.type];

  switch (this.type) {
    case 1:
      var points = [];

      for (var i = 0; i < coords.length; i++) {
        points[i] = coords[i][0];
      }

      coords = points;
      project(coords, x0, y0, size);
      break;

    case 2:
      for (var _i = 0; _i < coords.length; _i++) {
        project(coords[_i], x0, y0, size);
      }

      break;

    case 3:
      coords = classifyRings(coords);

      for (var _i2 = 0; _i2 < coords.length; _i2++) {
        for (var j = 0; j < coords[_i2].length; j++) {
          project(coords[_i2][j], x0, y0, size);
        }
      }

      break;

    default:
  }

  if (coords.length === 1) {
    coords = coords[0];
  } else {
    type = "Multi".concat(type);
  }

  var result = {
    type: 'Feature',
    geometry: {
      type: type,
      coordinates: coords
    },
    properties: this.properties
  };

  if ('id' in this) {
    result.id = this.id;
  }

  return result;
}

function readPBF(file, options) {
  var vectorTile = new _vectorTile.VectorTile(new _pbf.default(file));
  var extentSource = options.extentSource || file.coords;
  var layers = Object.keys(vectorTile.layers);

  if (layers.length < 1) {
    return;
  }

  var crsInId = Number(options.crsIn.slice(5)); // We need to create a featureCollection as VectorTile does no support it

  var geojson = {
    type: 'FeatureCollection',
    features: [],
    crs: {
      type: 'EPSG',
      properties: {
        code: crsInId
      }
    }
  };
  layers.forEach(function (layer_id) {
    var l = vectorTile.layers[layer_id];

    for (var i = 0; i < l.length; i++) {
      var feature = void 0; // We need to move from TMS to Google/Bing/OSM coordinates
      // https://alastaira.wordpress.com/2011/07/06/converting-tms-tile-coordinates-to-googlebingosm-tile-coordinates/
      // Only if the layer.origin is top

      if (options.isInverted) {
        feature = toGeoJSON.bind(l.feature(i))(extentSource.col, extentSource.row, extentSource.zoom);
      } else {
        var y = 1 << extentSource.zoom;
        feature = toGeoJSON.bind(l.feature(i))(extentSource.col, y - extentSource.row - 1, extentSource.zoom);
      }

      if (layers.length > 1) {
        feature.properties.vt_layer = layer_id;
      }

      geojson.features.push(feature);
    }
  });
  return _GeoJsonParser.default.parse(geojson, {
    crsIn: options.crsIn,
    crsOut: options.crsOut,
    filteringExtent: options.filteringExtent,
    filter: options.filter,
    withNormal: options.withNormal,
    withAltitude: options.withAltitude,
    mergeFeatures: options.mergeFeatures,
    buildExtent: true
  }).then(function (f) {
    f.extent.zoom = extentSource.zoom;
    return f;
  });
}
/**
 * @module VectorTileParser
 */


var _default = {
  /**
   * Parse a vector tile file and return a [Feature]{@link module:GeoJsonParser.Feature}
   * or an array of Features. While multiple formats of vector tile are
   * available, the only one supported for the moment is the
   * [Mapbox Vector Tile]{@link https://www.mapbox.com/vector-tiles/specification/}.
   *
   * @param {ArrayBuffer} file - The vector tile file to parse.
   * @param {Object} options - Options controlling the parsing.
   * @param {Extent} options.extent - The Extent to convert the input coordinates to.
   * @param {Extent} options.coords - Coordinates of the layer.
   * @param {Extent=} options.filteringExtent - Optional filter to reject features
   * outside of this extent.
   * @param {boolean} [options.mergeFeatures=true] - If true all geometries are merged by type and multi-type
   * @param {boolean} [options.withNormal=true] - If true each coordinate normal is computed
   * @param {boolean} [options.withAltitude=true] - If true each coordinate altitude is kept
   * @param {function=} options.filter - Filter function to remove features.
   * @param {string=} options.isInverted - This option is to be set to the
   * correct value, true or false (default being false), if the computation of
   * the coordinates needs to be inverted to same scheme as OSM, Google Maps
   * or other system. See [this link]{@link
   * https://alastaira.wordpress.com/2011/07/06/converting-tms-tile-coordinates-to-googlebingosm-tile-coordinates}
   * for more informations.
   *
   * @return {Promise} A Promise resolving with a Feature or an array a
   * Features.
   */
  parse: function parse(file, options) {
    return Promise.resolve(readPBF(file, options));
  }
};
exports.default = _default;